"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const node_fetch_1 = __importDefault(require("node-fetch"));
const execa_1 = __importDefault(require("execa"));
const fs_1 = require("fs");
const build_utils_1 = require("@now/build-utils");
// portable ruby from https://portableruby.com/ for bootstrapping
// ruby deployment on nodejs8.10. version 2.5.3 matches ruby2.5.
const url = 'https://portableruby.s3.amazonaws.com/ruby-2.5.3.zip';
// downloads `ruby` and `gem` and returns their absolute paths
async function downloadPortableRuby(path) {
    console.log('downloading "ruby-2.5.3.zip"...');
    const res = await node_fetch_1.default(url);
    if (!res.ok || res.status !== 200) {
        throw new Error(`Could not download "ruby-2.5.3.zip" from "${url}"`);
    }
    const dir = await build_utils_1.getWriteableDirectory();
    const filePath = path_1.join(dir, 'ruby-2.5.3.zip');
    const writeStream = fs_1.createWriteStream(filePath);
    await new Promise((resolve, reject) => {
        res.body
            .on('error', reject)
            .pipe(writeStream)
            .on('finish', resolve);
    });
    try {
        await execa_1.default('unzip', ['-q', '-d', path, filePath], { stdio: 'inherit' });
    }
    catch (err) {
        console.log('could not unzip ruby-2.5.3.zip');
        throw err;
    }
    return {
        rubyPath: path_1.join(path, 'bin', 'ruby'),
        gemPath: path_1.join(path, 'bin', 'gem'),
    };
}
// downloads and installs `bundler` (respecting
// process.env.GEM_HOME), and returns
// the absolute path to it
async function downloadAndInstallBundler() {
    const { GEM_HOME } = process.env;
    if (!GEM_HOME) {
        // this is the directory in which `gem` will be
        // installed to. `--prefix` will assume `~` if this
        // is not set, and `~` is not writeable on AWS Lambda.
        // let's refuse to proceed
        throw new Error('Could not install "bundler": "GEM_HOME" env var is not set');
    }
    console.log('installing ruby...');
    const { rubyPath, gemPath } = await downloadPortableRuby(GEM_HOME);
    console.log('installing bundler...');
    await execa_1.default(gemPath, ['install', 'bundler', '--no-document'], { stdio: 'inherit' });
    return {
        rubyPath,
        gemPath,
        bundlerPath: path_1.join(GEM_HOME, 'bin', 'bundler'),
    };
}
exports.downloadAndInstallBundler = downloadAndInstallBundler;
